'use strict';

/**
 * Create a new ShakeLauncher instance, which shakes when it launches.
 * @param {Game} game - The game in which the launcher is placed
 * @param {Number} x - The launcher's starting x-coordinate
 * @param {Number} y - The launcher's starting y-coordinate
 */
function ShakeLauncher(game, x, y) {
	Launcher.call(this, game, x, y);
	
	this._textureFrame = new Rectangle(0, 0, this.FRAME_WIDTH, this.FRAME_HEIGHT);
	this._body.texture.frame = this._textureFrame;
	
	this._body.anchor.x = this.ROTATION_X;
	this._body.anchor.y = this.ROTATION_Y;
	
	game.app.stage.addChild(this._body);
}
// Extend Launcher.
ShakeLauncher.prototype = Object.create(Launcher.prototype);

// Define constants.
/** @constant {Number} The width of each frame of the launcher animation */
ShakeLauncher.prototype.FRAME_WIDTH = 170;
/** @constant {Number} The height of each frame of the launcher animation */
ShakeLauncher.prototype.FRAME_HEIGHT = 170;
/** @constant {Number} The x-position of the launcher's rotation anchor point, from 0 to 1 */
ShakeLauncher.prototype.ROTATION_X = 0.1;
/** @constant {Number} The y-position of the launcher's rotation anchor point, from 0 to 1 */
ShakeLauncher.prototype.ROTATION_Y = 0.7;
/** @constant {Number} The starting rotation of the launcher, in radians */
ShakeLauncher.prototype.START_ROTATION = 0;
/** @constant {Number} The end rotation of the launcher when it launches, in radians */
ShakeLauncher.prototype.LAUNCH_ROTATION = -0.1 * Math.TAU;

/**
 * Queue assets to be preloaded that might be needed by any Launcher instance.
 */
ShakeLauncher.queueAssets = Launcher.queueAssets.bind(ShakeLauncher);

/**
 * @override
 * Update the launcher for the next frame.
 * @param {Number} deltaTime - The time since the last frame, in milliseconds
 */
ShakeLauncher.prototype.update = function (deltaTime) {
	Launcher.prototype.update.call(this, deltaTime);
	
	if (this._state === this.STATES.LAUNCH) {
		if (this._currentFrame >= this.LAUNCH_DURATION) {
			return;
		}
		this._body.rotation = Utils.mappedAbsFunc(this._currentFrame, this.LAUNCH_DURATION, this.START_ROTATION, this.LAUNCH_ROTATION);
	}
};
